@@ -1,25 +1,21 @@
-;; guile-lib
+;; (texinfo) -- parsing of texinfo into SXML
 ;; Copyright (C) 2004 Andy Wingo <wingo at pobox dot com>
 ;; Copyright (C) 2001,2002 Oleg Kiselyov <oleg at pobox dot com>
 
-;; This file is based on SSAX's SSAX.scm.
-
-;; This program is free software; you can redistribute it and/or    
-;; modify it under the terms of the GNU General Public License as   
-;; published by the Free Software Foundation; either version 2 of   
-;; the License, or (at your option) any later version.              
-;;                                                                  
-;; This program is distributed in the hope that it will be useful,  
-;; but WITHOUT ANY WARRANTY; without even the implied warranty of   
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    
-;; GNU General Public License for more details.                     
-;;                                                                  
-;; You should have received a copy of the GNU General Public License
-;; along with this program; if not, contact:
+;; This program is free software: you can redistribute it and/or modify
+;; it under the terms of the GNU General Public License as published by
+;; the Free Software Foundation, either version 3 of the License, or
+;; (at your option) any later version.
 ;;
-;; Free Software Foundation           Voice:  +1-617-542-5942
-;; 59 Temple Place - Suite 330        Fax:    +1-617-542-2652
-;; Boston, MA  02111-1307,  USA       gnu@gnu.org
+;; This program is distributed in the hope that it will be useful,
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+;; GNU General Public License for more details.
+;;
+;; You should have received a copy of the GNU General Public License
+;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
+;; This file is based on SSAX's SSAX.scm.
 
 ;;; Commentary:
 ;;
@@ -239,6 +235,7 @@ lambda. Only present for @code{INLINE-ARGS}, @code{EOL-ARGS},
     (bullet             INLINE-ARGS . ())
     (copyright          INLINE-ARGS . ())
     (tie                INLINE-ARGS . ())
+    (image              INLINE-ARGS . (file #:opt width height alt-text extension))
 
     ;; EOL args elements
     (node               EOL-ARGS . (name #:opt next previous up))
