@@ -1,22 +1,18 @@
-;; guile-lib
-;; Copyright (C) 2003,2004 Andy Wingo <wingo at pobox dot com>
+;; (texinfo reflection) -- documenting Scheme as stexinfo
+;; Copyright (C) 2003,2004,2010  Andy Wingo <wingo at pobox dot com>
 
-;; This program is free software; you can redistribute it and/or    
-;; modify it under the terms of the GNU General Public License as   
-;; published by the Free Software Foundation; either version 2 of   
-;; the License, or (at your option) any later version.              
-;;                                                                  
-;; This program is distributed in the hope that it will be useful,  
-;; but WITHOUT ANY WARRANTY; without even the implied warranty of   
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    
-;; GNU General Public License for more details.                     
-;;                                                                  
-;; You should have received a copy of the GNU General Public License
-;; along with this program; if not, contact:
+;; This program is free software: you can redistribute it and/or modify
+;; it under the terms of the GNU General Public License as published by
+;; the Free Software Foundation, either version 3 of the License, or
+;; (at your option) any later version.
 ;;
-;; Free Software Foundation           Voice:  +1-617-542-5942
-;; 59 Temple Place - Suite 330        Fax:    +1-617-542-2652
-;; Boston, MA  02111-1307,  USA       gnu@gnu.org
+;; This program is distributed in the hope that it will be useful,
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+;; GNU General Public License for more details.
+;;
+;; You should have received a copy of the GNU General Public License
+;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 ;;
@@ -36,16 +32,19 @@
   #:use-module (texinfo plain-text)
   #:use-module (srfi srfi-13)
   #:use-module (scheme kwargs)
-  #:use-module (scheme session)
+  #:use-module (ice-9 session)
   #:use-module (ice-9 documentation)
   #:use-module (ice-9 optargs)
   #:use-module ((sxml transform)
                 #:select (pre-post-order))
   #:export (module-stexi-documentation
+            script-stexi-documentation
             object-stexi-documentation
             package-stexi-standard-copying
             package-stexi-standard-titlepage
+            package-stexi-generic-menu
             package-stexi-standard-menu
+            package-stexi-extended-menu
             package-stexi-standard-prologue
             package-stexi-documentation))
 
@@ -123,14 +122,6 @@
    (else
     (process-args (and=> (procedure-source proc) cadr)))))
 
-;; like the normal false-if-exception, but doesn't affect the-last-stack
-(define-macro (false-if-exception exp)
-  `(catch #t
-          (lambda ()
-            (with-fluids ((the-last-stack (fluid-ref the-last-stack)))
-              ,exp))
-          (lambda args #f)))
-
 ;; This is really nasty, I wish guile gave a better way to get this...
 (define (get-macro-args macro)
   (process-args (false-if-exception
@@ -154,19 +145,41 @@
             (texi-fragment->stexi str)))
       `(*fragment* (verbatim ,str))))
 
+(define method-formals
+  (and (defined? 'method-formals) method-formals))
+
 (define (method-stexi-arguments method)
-  (define (arg-texinfo arg)
-    `(" (" (var ,(symbol->string (car arg))) " "
-      (code ,(symbol->string (cadr arg))) ")"))
-  (let lp ((bindings (cadr (method-source method))) (out '()))
-    (cond
-     ((null? bindings)
-      (reverse out))
-     ((not (pair? (car bindings)))
-      (append (reverse out) (arg-texinfo bindings) (list "...")))
-     (else
-      (lp (cdr bindings)
-          (append (reverse (arg-texinfo (car bindings))) out))))))
+  (cond
+   (method-formals
+    (let lp ((formals (method-formals method))
+             (specializers (method-specializers method))
+             (out '()))
+      (define (arg-texinfo formal specializer)
+        `(" (" (var ,(symbol->string formal)) " "
+          (code ,(symbol->string (class-name specializer))) ")"))
+      (cond
+       ((null? formals) (reverse out))
+       ((pair? formals)
+        (lp (cdr formals) (cdr specializers)
+            (append (reverse (arg-texinfo (car formals) (car specializers)))
+                    out)))
+       (else
+        (append (reverse out) (arg-texinfo formals specializers)
+                (list "..."))))))
+   ((method-source method)
+    (let lp ((bindings (cadr (method-source method))) (out '()))
+      (define (arg-texinfo arg)
+        `(" (" (var ,(symbol->string (car arg))) " "
+          (code ,(symbol->string (cadr arg))) ")"))
+      (cond
+       ((null? bindings)
+        (reverse out))
+       ((not (pair? (car bindings)))
+        (append (reverse out) (arg-texinfo bindings) (list "...")))
+       (else
+        (lp (cdr bindings)
+            (append (reverse (arg-texinfo (car bindings))) out))))))
+   (else (warn method) '())))
 
 (define/kwargs (object-stexi-documentation object (name "[unknown]")
                                            (force #f))
@@ -294,17 +307,34 @@ documentation will be formatted as @code{stexi}
               (section "Usage")
               ,@(apply append! (make-defs)))))
 
-(define (stexi-help-handler name value)
-  (stexi->plain-text (object-stexi-documentation value name #:force #t)))
+(define (script-stexi-documentation scriptpath)
+  "Return documentation for given script. The documentation will be
+taken from the script's commentary, and will be returned in the
+@code{stexi} format (@pxref{texinfo,texinfo})."
+  (let ((commentary (file-commentary scriptpath)))
+    `(texinfo (% (title ,(basename scriptpath)))
+              (node (% (name ,(basename scriptpath))))
+              ,@(if commentary
+                    (cdr
+                     (string->stexi
+                      (string-trim-both commentary #\newline)))
+                    '()))))
 
-(add-value-help-handler! stexi-help-handler)
+(cond
+ ((defined? 'add-value-help-handler!)
+  (add-value-help-handler! 
+   (lambda (name value)
+     (stexi->plain-text
+      (object-stexi-documentation value name #:force #t))))
+  (add-name-help-handler!
+   (lambda (name)
+    (and (list? name)
+         (and-map symbol? name)
+         (stexi->plain-text (module-stexi-documentation name)))))))
 
-(define (module-help-handler name)
-  (and (list? name)
-       (and-map symbol? name)
-       (stexi->plain-text (module-stexi-documentation name))))
-
-(add-name-help-handler! module-help-handler)
+;; we could be dealing with an old (ice-9 session); fondle it to get
+;; module-commentary
+(define module-commentary (@@ (ice-9 session) module-commentary))
 
 (define (package-stexi-standard-copying name version updated years
                                         copyright-holder permissions)
@@ -351,10 +381,10 @@ Here is an example of the usage of this procedure:
      (vskip (% (all "0pt plus 1filll")))
      (insertcopying))))
 
-(define (package-stexi-standard-menu name modules module-descriptions
-                                     extra-entries)
-  "Create a standard top node and menu, suitable for processing
-by makeinfo."
+(define (package-stexi-generic-menu name entries)
+  "Create a menu from a generic alist of entries, the car of which
+should be the node name, and the cdr the description. As an exception,
+an entry of @code{#f} will produce a separator."
   (define (make-entry node description)
     `("* " ,node "::"
       ,(make-string (max (- 21 (string-length node)) 2) #\space)
@@ -367,18 +397,46 @@ by makeinfo."
       ,@(apply
          append
          (map
-          (lambda (module description)
-            (make-entry (module-name->node-name module) description))
-          modules module-descriptions))
-      ,@(if extra-entries
-            (cons "\n" 
-                  (apply append (map make-entry
-                                     (map car extra-entries)
-                                     (map cdr extra-entries))))
-            '())))
+          (lambda (entry)
+            (if entry
+                (make-entry (car entry) (cdr entry))
+                '("\n")))
+          entries))))
     (iftex
      (shortcontents))))
 
+
+(define (package-stexi-standard-menu name modules module-descriptions
+                                     extra-entries)
+  "Create a standard top node and menu, suitable for processing
+by makeinfo."
+  (package-stexi-generic-menu
+   name
+   (let ((module-entries (map cons
+                              (map module-name->node-name modules)
+                              module-descriptions))
+         (separate-sections (lambda (x) (if (null? x) x (cons #f x)))))
+     `(,@module-entries
+       ,@(separate-sections extra-entries)))))
+
+(define (package-stexi-extended-menu name module-pairs script-pairs
+                                     extra-entries)
+  "Create an \"extended\" menu, like the standard menu but with a
+section for scripts."
+  (package-stexi-generic-menu
+   name
+   (let ((module-entries (map cons
+                              (map module-name->node-name
+                                   (map car module-pairs))
+                              (map cdr module-pairs)))
+         (script-entries (map cons
+                              (map basename (map car script-pairs))
+                              (map cdr script-pairs)))
+         (separate-sections (lambda (x) (if (null? x) x (cons #f x)))))
+     `(,@module-entries
+       ,@(separate-sections script-entries)
+       ,@(separate-sections extra-entries)))))
+
 (define (package-stexi-standard-prologue name filename category
                                          description copying titlepage
                                          menu)
@@ -400,12 +458,9 @@ package-stexi-standard-menu,package-stexi-standard-menu}."
     ,@titlepage
     ,@menu))
 
-(define (package-stexi-documentation-helper sym-name args)
-  ;; returns a list of forms
+(define (stexi->chapter stexi)
   (pre-post-order
-   (apply module-stexi-documentation sym-name args)
-   ;; here taking advantage of the fact that we know what
-   ;; module-stexi-documentation outputs
+   stexi
    `((texinfo . ,(lambda (tag attrs node . body)
                    `(,node
                      (chapter ,@(assq-ref (cdr attrs) 'title))
@@ -416,7 +471,8 @@ package-stexi-standard-menu,package-stexi-standard-menu}."
 (define/kwargs (package-stexi-documentation modules name filename
                                             prologue epilogue
                                             (module-stexi-documentation-args
-                                             '()))
+                                             '())
+                                            (scripts '()))
   "Create stexi documentation for a @dfn{package}, where a
 package is a set of modules that is released together.
 
@@ -449,9 +505,14 @@ useful to define a @code{#:docs-resolver} argument."
        (filename ,filename))
     ,@prologue
     ,@(append-map (lambda (mod)
-                    (package-stexi-documentation-helper
-                     mod module-stexi-documentation-args))
+                    (stexi->chapter
+                     (apply module-stexi-documentation
+                            mod module-stexi-documentation-args)))
                   modules)
+    ,@(append-map (lambda (script)
+                    (stexi->chapter
+                     (script-stexi-documentation script)))
+                  scripts)
     ,@epilogue))
 
 ;;; arch-tag: bbe2bc03-e16d-4a9e-87b9-55225dc9836c
